package com.kwai.allin.demo.ui

import android.content.Context
import android.graphics.Color
import android.os.Looper
import android.os.SystemClock
import android.util.AttributeSet
import android.util.Log
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import android.widget.ListView
import android.widget.TextView
//import android.support.constraint.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintLayout
import com.kwai.allin.demo.R
import com.kwai.sdk.KwaiSdk
import com.kwai.sdk.combus.cloudgame.CloudRunMode
import java.text.DateFormat
import java.text.SimpleDateFormat
import java.util.*
import kotlin.math.abs

class ConsoleListView : ConstraintLayout {
  var sbLog = StringBuffer()
  var mList: ArrayList<String> = ArrayList()
  var mListCache: ArrayList<String> = ArrayList() // listview不可见时候缓存 可见后同步到mList
  var mTopBarTv: TextView? = null
  var mListView: ListView? = null
  var mToggleBtn: View? = null
  var mUnreadCnt: TextView? = null

  var mRunnable: Runnable? = null
  var createTs = SystemClock.elapsedRealtime()

  constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)
  constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs,
      defStyleAttr) {
    inflate(context, R.layout.console_log_view, this)
    mTopBarTv = findViewById(R.id.log_tip_tv)
    mListView = findViewById(R.id.log_list_view)
    mListView?.cacheColorHint = 0
    mListView?.divider = null
    mListView?.dividerHeight = 0
    mListView?.transcriptMode = ListView.TRANSCRIPT_MODE_ALWAYS_SCROLL
    mListView?.adapter = object : BaseAdapter() {
      override fun getCount(): Int {
        return mList.size
      }

      override fun getItem(position: Int): Any {
        return mList[position]
      }

      override fun getItemId(position: Int): Long {
        return position.toLong()
      }

      override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
        var convertView: View? = convertView
        if (convertView == null) {
          convertView = TextView(parent.context)
          convertView.setTextIsSelectable(true)
          convertView.setTextColor(Color.WHITE)
        }
        (convertView as TextView).text = mList[position]
        return convertView
      }
    }
    mToggleBtn = findViewById(R.id.log_toggle_btn)
    mUnreadCnt = findViewById(R.id.log_unread_cnt)
    mToggleBtn?.setOnClickListener {
      toggleclick()
    }
    mRunnable = Runnable {
      if (visibility != VISIBLE) {
        return@Runnable
      }
      if (mTopBarTv?.visibility == VISIBLE) {
        mTopBarTv?.text = "LogInfo：" + formatDuration() +
            KwaiSdk.getSdkInfo() +
            "  云模式:" + if (CloudRunMode.isOn()) "on" else "off"
      }
      postOnAnimationDelayed(mRunnable, 1000)
    }
  }

  override fun onAttachedToWindow() {
    super.onAttachedToWindow()
    postOnAnimation(mRunnable)
  }

  override fun onDetachedFromWindow() {
    super.onDetachedFromWindow()
    removeCallbacks(mRunnable)
  }

  fun toggleclick() {
    if (mListView?.visibility != VISIBLE) {
      mTopBarTv?.visibility = VISIBLE
      mListView?.visibility = VISIBLE
      if (mListCache.size > 0) {
        mList.addAll(mListCache)
        (mListView?.adapter as BaseAdapter?)?.notifyDataSetChanged()
      }
      mListCache.clear()
      updateBtnCnt()
    } else {
      mTopBarTv?.visibility = GONE
      mListView?.visibility = GONE
    }
  }


  fun showLog(logLine: String?) {
    if (logLine != null) {
      Log.d("日志:", logLine)
    }
    if (Looper.myLooper() == Looper.getMainLooper()) {
      showLog(logLine, false)
    } else {
      post {
        showLog(logLine, false)
      }

    }
  }

  private fun ts(): String {
    val format: DateFormat = SimpleDateFormat("MM-dd hh:mm ssSSS", Locale.getDefault())
    val time = format.format(Date())
    return "$time:"
  }

  private fun updateBtnCnt() {
    if (mListView?.visibility != VISIBLE && mListCache.size > 0) {
      mUnreadCnt?.visibility = VISIBLE
      mUnreadCnt?.text = mListCache.size.coerceAtMost(99).toString()
      return
    }
    mUnreadCnt?.visibility = INVISIBLE
  }

  private fun formatDuration(): String? {
    val absSeconds = abs(SystemClock.elapsedRealtime() - createTs) /1000
    val positive = String.format(
        "%d:%02d:%02d",
        absSeconds / 3600,
        absSeconds % 3600 / 60,
        absSeconds % 60)
    return if (absSeconds < 0) "-$positive" else positive
  }

  private fun showLog(logLine: String?, withTs: Boolean) {
    if (!withTs) {
      sbLog.append(ts()).append(logLine)
    } else {
      sbLog.append(logLine)
    }
    if (mListView?.visibility == VISIBLE) {
      mList.add(sbLog.toString())
      (mListView?.adapter as BaseAdapter?)?.notifyDataSetChanged()
    } else {
      mListCache.add(sbLog.toString())
      updateBtnCnt()
    }
    sbLog.delete(0, sbLog.length)
  }
}