package com.kwai.allin.demo.util;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.os.Build;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;


public class NotchUtils {
  private static final String TAG = NotchUtils.class.getSimpleName();

  public static boolean hasNotchInScreen(Context context) {
    if (hasNotchInScreenForHuawei(context)) {
      return true;
    } else if (hasNotchInScreenForXiaoMi()) {
      return true;
    } else if (hasNotchInScreenForOppo(context)) {
      return true;
    } else if (hasNotchInScreenForVoio(context)) {
      return true;
    } else if (hasNotchInScreenLenovo(context)) {
      return true;
    }
    return false;
  }

  private static boolean hasNotchInScreenLenovo(Context context) {
    int identifier =
        context.getResources().getIdentifier("config_screen_has_notch", "bool", "android");
    if (identifier == 0) {
      return false;
    }
    boolean aBoolean = context.getResources().getBoolean(identifier);
    return aBoolean;
  }

  private static boolean hasNotchInScreenForHuawei(Context context) {
    try {
      ClassLoader cl = context.getClassLoader();
      Class HwNotchSizeUtil = cl.loadClass("com.huawei.android.util.HwNotchSizeUtil");
      Method get = HwNotchSizeUtil.getMethod("hasNotchInScreen");
      return (boolean) get.invoke(HwNotchSizeUtil);
    } catch (Exception e) {
    }
    return false;
  }

  private static boolean hasNotchInScreenForOppo(Context context) {
    return context.getPackageManager().hasSystemFeature("com.oppo.feature.screen.heteromorphism");
  }

  private static boolean hasNotchInScreenForVoio(Context context) {
    try {
      ClassLoader cl = context.getClassLoader();
      Class FtFeature = cl.loadClass("com.util.FtFeature");
      Method get = FtFeature.getMethod("isFeatureSupport", int.class);
      return (boolean) get.invoke(FtFeature, 0x00000020);
    } catch (Exception e) {

    }
    return false;
  }

  private static boolean hasNotchInScreenForXiaoMi() {
    try {
      Class<?> cls = Class.forName("android.os.SystemProperties");
      Method get = cls.getDeclaredMethod("get", String.class, String.class);
      String notch = (String) get.invoke(null, "ro.miui.notch", "0");
      return (!TextUtils.isEmpty(notch) && (notch.equals("1")));
    } catch (Exception e) {
      Log.e(TAG, "hasNotchInScreenForXiaoMi e=" + e.getMessage());
    }
    return false;
  }



  public static void fullScreen(Activity Awindow) {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
      int visibility = View.SYSTEM_UI_FLAG_LAYOUT_STABLE | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;
      boolean hasNotchInScreen = hasNotchInScreen(Awindow);
      if (hasNotchInScreen) {
        visibility |= View.SYSTEM_UI_FLAG_FULLSCREEN;
      } else {
        visibility |= View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN;
      }
      Window window = Awindow.getWindow();
      window.getDecorView().setSystemUiVisibility(visibility);
      window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
      window.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION);
      window.setStatusBarColor(Color.TRANSPARENT);
      /**
       * 针对刘海屏的适配代码对于小米O设备未进行适配
       */
      WindowManager.LayoutParams params = window.getAttributes();

      //编译不过？target sdk 26?????
      if (Build.VERSION.SDK_INT >= 28) {
        try {
          Field layoutInDisplayCutoutMode = params.getClass().getField("layoutInDisplayCutoutMode");
          Field layoutInDisplayCutoutModeShortEdges =
              params.getClass().getField("LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES");
          layoutInDisplayCutoutModeShortEdges.setAccessible(true);
          Object o = layoutInDisplayCutoutModeShortEdges.get(WindowManager.LayoutParams.class);
          layoutInDisplayCutoutMode.set(params, o);
        } catch (NoSuchFieldException | IllegalAccessException e) {
          e.printStackTrace();
        }
//                params.layoutInDisplayCutoutMode = WindowManager.LayoutParams
//                .LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES;
      }
      window.setAttributes(params);
      if (!hasNotchInScreen) {
        Awindow.getWindow().getDecorView().setSystemUiVisibility(
            Awindow.getWindow().getDecorView().getSystemUiVisibility() |
                View.SYSTEM_UI_FLAG_FULLSCREEN);
      }
    }
  }
}
