package com.kwai.allin.demo.util;

import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.DSAPublicKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.content.pm.PackageManager;
import android.content.pm.Signature;
import android.database.Cursor;
import android.net.Uri;
import android.provider.MediaStore;
import android.text.TextUtils;
import android.util.Log;

import com.kwai.middleware.azeroth.utils.DigestUtils;
import com.kwai.sdk.subbus.pay.model.KwaiPayInfo;

public class Util {

  public static final String KEY_ALGORITHM = "RSA";
  /**
   * 签名算法
   */
  public static final String SIGN_ALGORITHMS = "SHA512WithRSA";

  /**
   * 下单签名参数
   */
  public static final String[] keys = new String[]{
      "app_id", "channel_id", "game_id",
      "role_id", "role_name", "role_level",
      "server_id", "server_name",
      "product_id", "product_name", "product_desc",
      "money", "currency_type",
      "notify_url", "user_ip", "extension",
      "third_party_trade_no"
  };

  public static String getSignSrc(String appId, String channel, String gameid, KwaiPayInfo info) {
    HashMap<String, String> map = new HashMap<String, String>();
    map.put("app_id", appId);
    map.put("channel_id", channel);
    map.put("role_id", info.roleId);
    map.put("role_name", info.roleName);
    map.put("role_level", info.roleLevel);
    map.put("server_id", info.serverId);
    map.put("server_name", info.serverName);
    map.put("product_id", info.productId);
    map.put("product_name", info.productName);
    map.put("product_desc", info.productDesc);
    map.put("money", String.valueOf(info.price));
    map.put("currency_type", info.currencyType);
    map.put("third_party_trade_no", info.orderId);
    map.put("notify_url", info.notifyUrl);
    map.put("user_ip", info.userIp);
    map.put("extension", info.extension);
    map.put("game_id", gameid);
    map.put("sign", info.sign);//此处有服务端计算得出
    return getSignStringFromMap(map);
  }

  public static String getSignStringFromMap(Map<String, String> map) {
    List<String> keysList = Arrays.asList(keys);
    List<String> list = new ArrayList<String>();
    for (Map.Entry<String, String> entry : map.entrySet()) {
      if (TextUtils.isEmpty(entry.getValue())) {
        continue;
      }
      if (keysList.contains(entry.getKey())) {
        list.add(entry.getKey() + "=" + entry.getValue());
      } else {
        System.out.println("all_" + entry.getKey() + " = " + entry.getValue());
      }
    }
    Collections.sort(list);
    return TextUtils.join("&", list);
  }

  /**
   * RSA签名
   *
   * @param content    待签名数据
   * @param privateKey 商户私钥
   * @param encode     字符集编码
   * @return 签名值
   */
  public static String sign(String content, String privateKey, String encode) {
    try {
      PKCS8EncodedKeySpec priPKCS8 =
          new PKCS8EncodedKeySpec(Base64.decode(privateKey, Base64.DEFAULT));
      KeyFactory keyf = KeyFactory.getInstance(KEY_ALGORITHM);
      PrivateKey priKey = keyf.generatePrivate(priPKCS8);
      java.security.Signature signature = java.security.Signature.getInstance(SIGN_ALGORITHMS);

      signature.initSign(priKey);
      signature.update(content.getBytes(encode));

      byte[] signed = signature.sign();

      return Base64.encodeToString(signed, Base64.DEFAULT).replaceAll("(\r\n|\r|\n|\n\r)", "");
    } catch (Exception e) {
      e.printStackTrace();
      throw new RuntimeException(e);
    }

  }

  public static String sign(String content, String privateKey) {
    return sign(content, privateKey, "utf-8");
  }

  /**
   * RSA验签名检查
   *
   * @param content   待签名数据
   * @param sign      签名值
   * @param publicKey 分配给开发商公钥
   * @param encode    字符集编码
   * @return 布尔值
   */
  public static boolean check(String content, String sign, String publicKey, String encode) {
    try {
      KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
      byte[] encodedKey = Base64.decode(publicKey, Base64.DEFAULT);
      PublicKey pubKey = keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey));
      boolean isRight = checkPublic(pubKey);
      if (!isRight) {
        return false;
      }
      java.security.Signature signature = java.security.Signature.getInstance(SIGN_ALGORITHMS);

      signature.initVerify(pubKey);
      signature.update(content.getBytes(encode));
      return signature.verify(Base64.decode(sign, Base64.DEFAULT));

    } catch (Exception e) {
      e.printStackTrace();
    }

    return false;
  }

  public static boolean check(String content, String sign, String publicKey) {
    return check(content, sign, publicKey, "utf-8");
  }

  public static Map<String, String> getMapFromUrl(String url) {
    Map<String, String> map = new HashMap<String, String>();
    if (!TextUtils.isEmpty(url)) {
      try {
        String src = url.contains("+") ? url : URLDecoder.decode(url, "utf-8");
        String[] kvs = src.split("&");
        for (String entry : kvs) {
          String[] kv = entry.split("=");
          if (kv.length > 1) {
            map.put(kv[0], kv[1]);
          }
        }
      } catch (UnsupportedEncodingException e) {
        e.printStackTrace();
      }
    }

    return map;
  }

  public static Map<String, String> json2Map(String src) {
    HashMap<String, String> map = new HashMap<String, String>();
    JSONObject json = null;
    try {
      json = new JSONObject(src);
      Iterator<String> jsonKeys = json.keys();
      while (jsonKeys.hasNext()) {
        String key = jsonKeys.next();
        try {
          map.put(key, String.valueOf(json.get(key)));
        } catch (JSONException e) {
          e.printStackTrace();
        }
      }
    } catch (JSONException e) {
      e.printStackTrace();
    }
    return map;
  }

  /**
   * 下单参数校验
   */
  public static String checkParams(Map<String, String> map) {
    StringBuilder builder = new StringBuilder();
    for (String key : keys) {
      String value = map.get(key);
      Log.d("params:", key + " = " + value);
      if (TextUtils.isEmpty(value)) {
        builder.append(key).append(" ");
      } else if (key.equals("money")) {
        int m;
        try {
          m = Integer.valueOf(map.get(key));
        } catch (Exception e) {
          e.printStackTrace();
          m = 0;
        }
        if (m == 0) {
          builder.append(key).append(" ");
        }
      }
    }
    Log.d("pay", "params lose:" + builder.toString());
    return builder.toString();
  }

  public static boolean checkPublic(PublicKey publickey) {
    String algorithm = publickey.getAlgorithm(); // 获取算法
    Log.d("check key", "" + algorithm);
    if (!"RSA".equals(algorithm)) {
      return false;
    }
    KeyFactory keyFact = null;
    try {
      keyFact = KeyFactory.getInstance(algorithm);
    } catch (NoSuchAlgorithmException e) {
      e.printStackTrace();
    }
    try {
      BigInteger prime = null;
      if ("RSA".equals(algorithm)) { // 如果是RSA加密
        RSAPublicKeySpec keySpec = null;
        keySpec = keyFact.getKeySpec(publickey, RSAPublicKeySpec.class);
        prime = keySpec.getModulus();

      } else if ("DSA".equals(algorithm)) { // 如果是DSA加密
        DSAPublicKeySpec keySpec = keyFact.getKeySpec(publickey, DSAPublicKeySpec.class);
        prime = keySpec.getP();
      }
      int len = prime.toString(2).length(); // 转换为二进制，获取公钥长度
      Log.d("check key", "length:" + len);
      return len == 4096;
    } catch (InvalidKeySpecException e) {
      e.printStackTrace();
    }
    return false;
  }

  public static class Base64 {

    public static final int DEFAULT = 0;

    public static final int NO_PADDING = 1;

    public static final int NO_WRAP = 2;

    public static final int CRLF = 4;

    public static final int URL_SAFE = 8;

    public static final int NO_CLOSE = 16;

    static abstract class Coder {
      public byte[] output;
      public int op;

      public abstract boolean process(byte[] input, int offset, int len, boolean finish);

      public abstract int maxOutputSize(int len);
    }

    public static byte[] decode(String str, int flags) {
      return decode(str.getBytes(), flags);
    }

    public static byte[] decode(byte[] input, int flags) {
      return decode(input, 0, input.length, flags);
    }

    public static byte[] decode(byte[] input, int offset, int len, int flags) {
      // Allocate space for the most data the input could represent.
      // (It could contain less if it contains whitespace, etc.)
      Decoder decoder = new Decoder(flags, new byte[len * 3 / 4]);

      if (!decoder.process(input, offset, len, true)) {
        throw new IllegalArgumentException("bad base-64");
      }

      // Maybe we got lucky and allocated exactly enough output space.
      if (decoder.op == decoder.output.length) {
        return decoder.output;
      }

      // Need to shorten the array, so allocate a new one of the
      // right size and copy.
      byte[] temp = new byte[decoder.op];
      System.arraycopy(decoder.output, 0, temp, 0, decoder.op);
      return temp;
    }

    static class Decoder extends Coder {

      private static final int[] DECODE = {
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
          52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -2, -1, -1,
          -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
          15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,
          -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
          41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      };

      private static final int[] DECODE_WEBSAFE = {
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1,
          52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -2, -1, -1,
          -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
          15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, 63,
          -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
          41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      };

      /** Non-data values in the DECODE arrays. */
      private static final int SKIP = -1;
      private static final int EQUALS = -2;

      private int state;   // state number (0 to 6)
      private int value;

      final private int[] alphabet;

      public Decoder(int flags, byte[] output) {
        this.output = output;

        alphabet = ((flags & URL_SAFE) == 0) ? DECODE : DECODE_WEBSAFE;
        state = 0;
        value = 0;
      }

      @Override
      public int maxOutputSize(int len) {
        return len * 3 / 4 + 10;
      }

      @Override
      public boolean process(byte[] input, int offset, int len, boolean finish) {
        if (this.state == 6) {
          return false;
        }

        int p = offset;
        len += offset;

        // Using local variables makes the decoder about 12%
        // faster than if we manipulate the member variables in
        // the loop.  (Even alphabet makes a measurable
        // difference, which is somewhat surprising to me since
        // the member variable is final.)
        int state = this.state;
        int value = this.value;
        int op = 0;
        final byte[] output = this.output;
        final int[] alphabet = this.alphabet;

        while (p < len) {
          // Try the fast path:  we're starting a new tuple and the
          // next four bytes of the input stream are all data
          // bytes.  This corresponds to going through states
          // 0-1-2-3-0.  We expect to use this method for most of
          // the data.
          //
          // If any of the next four bytes of input are non-data
          // (whitespace, etc.), value will end up negative.  (All
          // the non-data values in decode are small negative
          // numbers, so shifting any of them up and or'ing them
          // together will result in a value with its top bit set.)
          //
          // You can remove this whole block and the output should
          // be the same, just slower.
          if (state == 0) {
            while (p + 4 <= len &&
                (value = ((alphabet[input[p] & 0xff] << 18) |
                    (alphabet[input[p + 1] & 0xff] << 12) |
                    (alphabet[input[p + 2] & 0xff] << 6) |
                    (alphabet[input[p + 3] & 0xff]))) >= 0) {
              output[op + 2] = (byte) value;
              output[op + 1] = (byte) (value >> 8);
              output[op] = (byte) (value >> 16);
              op += 3;
              p += 4;
            }
            if (p >= len) {
              break;
            }
          }

          // The fast path isn't available -- either we've read a
          // partial tuple, or the next four input bytes aren't all
          // data, or whatever.  Fall back to the slower state
          // machine implementation.

          int d = alphabet[input[p++] & 0xff];

          switch (state) {
            case 0:
              if (d >= 0) {
                value = d;
                ++state;
              } else if (d != SKIP) {
                this.state = 6;
                return false;
              }
              break;

            case 1:
              if (d >= 0) {
                value = (value << 6) | d;
                ++state;
              } else if (d != SKIP) {
                this.state = 6;
                return false;
              }
              break;

            case 2:
              if (d >= 0) {
                value = (value << 6) | d;
                ++state;
              } else if (d == EQUALS) {
                // Emit the last (partial) output tuple;
                // expect exactly one more padding character.
                output[op++] = (byte) (value >> 4);
                state = 4;
              } else if (d != SKIP) {
                this.state = 6;
                return false;
              }
              break;

            case 3:
              if (d >= 0) {
                // Emit the output triple and return to state 0.
                value = (value << 6) | d;
                output[op + 2] = (byte) value;
                output[op + 1] = (byte) (value >> 8);
                output[op] = (byte) (value >> 16);
                op += 3;
                state = 0;
              } else if (d == EQUALS) {
                // Emit the last (partial) output tuple;
                // expect no further data or padding characters.
                output[op + 1] = (byte) (value >> 2);
                output[op] = (byte) (value >> 10);
                op += 2;
                state = 5;
              } else if (d != SKIP) {
                this.state = 6;
                return false;
              }
              break;

            case 4:
              if (d == EQUALS) {
                ++state;
              } else if (d != SKIP) {
                this.state = 6;
                return false;
              }
              break;

            case 5:
              if (d != SKIP) {
                this.state = 6;
                return false;
              }
              break;
          }
        }

        if (!finish) {
          // We're out of input, but a future call could provide
          // more.
          this.state = state;
          this.value = value;
          this.op = op;
          return true;
        }

        // Done reading input.  Now figure out where we are left in
        // the state machine and finish up.

        switch (state) {
          case 0:
            // Output length is a multiple of three.  Fine.
            break;
          case 1:
          case 4:
            // Read one padding '=' when we expected 2.  Illegal.
            // Read one extra input byte, which isn't enough to
            // make another output byte.  Illegal.
            this.state = 6;
            return false;
          case 2:
            // Read two extra input bytes, enough to emit 1 more
            // output byte.  Fine.
            output[op++] = (byte) (value >> 4);
            break;
          case 3:
            // Read three extra input bytes, enough to emit 2 more
            // output bytes.  Fine.
            output[op++] = (byte) (value >> 10);
            output[op++] = (byte) (value >> 2);
            break;
          case 5:
            // Read all the padding '='s we expected and no more.
            // Fine.
            break;
        }

        this.state = state;
        this.op = op;
        return true;
      }
    }

    public static String encodeToString(byte[] input, int flags) {
      return new String(encode(input, flags), StandardCharsets.US_ASCII);
    }

    public static String encodeToString(byte[] input, int offset, int len, int flags) {
      return new String(encode(input, offset, len, flags), StandardCharsets.US_ASCII);
    }

    public static byte[] encode(byte[] input, int flags) {
      return encode(input, 0, input.length, flags);
    }

    public static byte[] encode(byte[] input, int offset, int len, int flags) {
      Encoder encoder = new Encoder(flags, null);

      // Compute the exact length of the array we will produce.
      int output_len = len / 3 * 4;

      // Account for the tail of the data and the padding bytes, if any.
      if (encoder.do_padding) {
        if (len % 3 > 0) {
          output_len += 4;
        }
      } else {
        switch (len % 3) {
          case 0:
            break;
          case 1:
            output_len += 2;
            break;
          case 2:
            output_len += 3;
            break;
        }
      }

      // Account for the newlines, if any.
      if (encoder.do_newline && len > 0) {
        output_len += (((len - 1) / (3 * Encoder.LINE_GROUPS)) + 1) *
            (encoder.do_cr ? 2 : 1);
      }

      encoder.output = new byte[output_len];
      encoder.process(input, offset, len, true);

      assert encoder.op == output_len;

      return encoder.output;
    }

    /* package */ static class Encoder extends Coder {

      public static final int LINE_GROUPS = 19;

      private static final byte[] ENCODE = {
          'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
          'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
          'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
          'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/',
      };

      private static final byte[] ENCODE_WEBSAFE = {
          'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
          'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
          'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
          'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '_',
      };

      final private byte[] tail;
      /* package */ int tailLen;
      private int count;

      final public boolean do_padding;
      final public boolean do_newline;
      final public boolean do_cr;
      final private byte[] alphabet;

      public Encoder(int flags, byte[] output) {
        this.output = output;

        do_padding = (flags & NO_PADDING) == 0;
        do_newline = (flags & NO_WRAP) == 0;
        do_cr = (flags & CRLF) != 0;
        alphabet = ((flags & URL_SAFE) == 0) ? ENCODE : ENCODE_WEBSAFE;

        tail = new byte[2];
        tailLen = 0;

        count = do_newline ? LINE_GROUPS : -1;
      }

      @Override
      public int maxOutputSize(int len) {
        return len * 8 / 5 + 10;
      }

      @Override
      public boolean process(byte[] input, int offset, int len, boolean finish) {
        // Using local variables makes the encoder about 9% faster.
        final byte[] alphabet = this.alphabet;
        final byte[] output = this.output;
        int op = 0;
        int count = this.count;

        int p = offset;
        len += offset;
        int v = -1;

        // First we need to concatenate the tail of the previous call
        // with any input bytes available now and see if we can empty
        // the tail.

        switch (tailLen) {
          case 0:
            // There was no tail.
            break;

          case 1:
            if (p + 2 <= len) {
              // A 1-byte tail with at least 2 bytes of
              // input available now.
              v = ((tail[0] & 0xff) << 16) |
                  ((input[p++] & 0xff) << 8) |
                  (input[p++] & 0xff);
              tailLen = 0;
            }
            break;

          case 2:
            if (p + 1 <= len) {
              // A 2-byte tail with at least 1 byte of input.
              v = ((tail[0] & 0xff) << 16) |
                  ((tail[1] & 0xff) << 8) |
                  (input[p++] & 0xff);
              tailLen = 0;
            }
            break;
        }

        if (v != -1) {
          output[op++] = alphabet[(v >> 18) & 0x3f];
          output[op++] = alphabet[(v >> 12) & 0x3f];
          output[op++] = alphabet[(v >> 6) & 0x3f];
          output[op++] = alphabet[v & 0x3f];
          if (--count == 0) {
            if (do_cr) {
              output[op++] = '\r';
            }
            output[op++] = '\n';
            count = LINE_GROUPS;
          }
        }

        // At this point either there is no tail, or there are fewer
        // than 3 bytes of input available.

        // The main loop, turning 3 input bytes into 4 output bytes on
        // each iteration.
        while (p + 3 <= len) {
          v = ((input[p] & 0xff) << 16) |
              ((input[p + 1] & 0xff) << 8) |
              (input[p + 2] & 0xff);
          output[op] = alphabet[(v >> 18) & 0x3f];
          output[op + 1] = alphabet[(v >> 12) & 0x3f];
          output[op + 2] = alphabet[(v >> 6) & 0x3f];
          output[op + 3] = alphabet[v & 0x3f];
          p += 3;
          op += 4;
          if (--count == 0) {
            if (do_cr) {
              output[op++] = '\r';
            }
            output[op++] = '\n';
            count = LINE_GROUPS;
          }
        }

        if (finish) {
          // Finish up the tail of the input.  Note that we need to
          // consume any bytes in tail before any bytes
          // remaining in input; there should be at most two bytes
          // total.

          if (p - tailLen == len - 1) {
            int t = 0;
            v = ((tailLen > 0 ? tail[t++] : input[p++]) & 0xff) << 4;
            tailLen -= t;
            output[op++] = alphabet[(v >> 6) & 0x3f];
            output[op++] = alphabet[v & 0x3f];
            if (do_padding) {
              output[op++] = '=';
              output[op++] = '=';
            }
            if (do_newline) {
              if (do_cr) {
                output[op++] = '\r';
              }
              output[op++] = '\n';
            }
          } else if (p - tailLen == len - 2) {
            int t = 0;
            v = (((tailLen > 1 ? tail[t++] : input[p++]) & 0xff) << 10) |
                (((tailLen > 0 ? tail[t++] : input[p++]) & 0xff) << 2);
            tailLen -= t;
            output[op++] = alphabet[(v >> 12) & 0x3f];
            output[op++] = alphabet[(v >> 6) & 0x3f];
            output[op++] = alphabet[v & 0x3f];
            if (do_padding) {
              output[op++] = '=';
            }
            if (do_newline) {
              if (do_cr) {
                output[op++] = '\r';
              }
              output[op++] = '\n';
            }
          } else if (do_newline && op > 0 && count != LINE_GROUPS) {
            if (do_cr) {
              output[op++] = '\r';
            }
            output[op++] = '\n';
          }

          assert tailLen == 0;
          assert p == len;
        } else {
          // Save the leftovers in tail to be consumed on the next
          // call to encodeInternal.

          if (p == len - 1) {
            tail[tailLen++] = input[p];
          } else if (p == len - 2) {
            tail[tailLen++] = input[p];
            tail[tailLen++] = input[p + 1];
          }
        }

        this.op = op;
        this.count = count;

        return true;
      }
    }

    private Base64() { }   // don't instantiate
  }

  public static String getSignature(Context context) {
    String packageName;
    if (context == null || TextUtils.isEmpty(packageName = context.getPackageName())) {
      return null;
    }
    try {
      Signature[] signatures = context.getPackageManager().getPackageInfo(packageName,
          PackageManager.GET_SIGNATURES).signatures;
      if (signatures == null || signatures.length == 0) {
        return null;
      } else {
        return DigestUtils.md5Hex(signatures[0].toByteArray());
      }
    } catch (Exception e) {
      e.printStackTrace();
      return null;
    }
  }

  public static String parseFilePath(Context context, Uri uri) {
    String[] filePathColumn = new String[]{MediaStore.Images.Media.DATA};
    Cursor cursor = context.getContentResolver().query(uri, filePathColumn, null, null, null);
    if (cursor != null) {
      cursor.moveToFirst();
      int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
      String picturePath = cursor.getString(columnIndex);
      cursor.close();
      return picturePath;
    }
    return null;
  }
}
